import json
import csv
import os
import numpy as np


class EVALtask1(object):

    def __init__(self, pred_filename=None, gt_dirname=None, list_dirname=None,
                 subset='val'):
        if not pred_filename:
            raise IOError('Please input a valid prediction file.')
        if not gt_dirname:
            raise IOError('Please input a valid ground truth file.')
        if not list_dirname:
            raise IOError('Please input a valid directory.')
        if not ((subset == "val") or (subset == "test")):
            raise IOError('Please input subset as "val" or "test"')

        self.subset = subset
        self.pred_filename = pred_filename
        self.gt_dirname = gt_dirname
        self.list_dirname = list_dirname

        self.eval_mode_list = ["sgdet", "sgcls", "predcls"]
        self.ap = None
        # TODO verbose
        self.verbose = True
        #self.num_tp = {"sgdet": 0, "sgcls": 0, "predcls": 0}
        self.tp_10 = 0
        self.tp_20 = 0
        self.num_collection = 0
        
        
    def compute_one_video_id(self, pred_dic, gt_dic):
        """compute accuracy as score in one video.

        Parameters
        ----------
        pred_dic : dic
            dic of prediction in each video ID

        list_dirname : str
            gdic of ground truth in each video

        """

        eval_mode = self.eval_mode_list[1]

        for k, v in pred_dic.items():
            one_frame = k # str

            for one_pred in v:
                self.num_collection += 1
                is_not_done_checking = True

                for one_gt in gt_dic['labels']:
                    if one_gt['frame'] == int(one_frame):

                        #if eval_mode == "sgdet":
                        #    # compute IoU
                        #if eval_mode == "sgdet":
                        #    # compute IoU
                        if eval_mode == "sgcls":
                            gt_obj_class = [one_gt['subject']['name'], one_gt['object']['name']]
                            gt_rect = [one_gt['subject']['rect']['x'], one_gt['subject']['rect']['y'],
                                       one_gt['subject']['rect']['w'], one_gt['subject']['rect']['h'],
                                       one_gt['object']['rect']['x'], one_gt['object']['rect']['y'],
                                       one_gt['object']['rect']['w'], one_gt['object']['rect']['h']]
                            #gt_rect = [list(one_gt['subject']['rect'].values()), list(one_gt['object']['rect'].values)]
                            #pred_obj_class = [one_pred['subject']['name'], one_pred['object']['name']]
                            pred_rect = [one_pred['subject_rect']['x'], one_pred['subject_rect']['y'],
                                         one_pred['subject_rect']['w'], one_pred['subject_rect']['h'],
                                         one_pred['object_rect']['x'], one_pred['object_rect']['y'],
                                         one_pred['object_rect']['w'], one_pred['object_rect']['h']]
                            #pred_rect = [list(one_pred['subject']['rect'].values()), list(one_pred['object']['rect'].values())]

                            if (set(gt_rect) == set(pred_rect)) and is_not_done_checking:
                                is_not_done_checking = False
                                score_list = []
                                label_list = []
                                obj_class_list = []
                                for one_label in one_pred['labels']:
                                    score_list.append(one_label["score"])
                                    label_list.append(one_label["relation"])
                                    obj_class_list.append([one_label["subject"], one_label["object"]])
                                score_index = list(np.argsort(score_list))[::-1]
                                for k_10 in range(10):
                                    if set(label_list[score_index[k_10]]) == set(one_gt['relationships']) and \
                                            obj_class_list[score_index[k_10]] == gt_obj_class:
                                        self.tp_10 += 1
                                        break
                                for k_20 in range(20):
                                    if set(label_list[score_index[k_20]]) == set(one_gt['relationships']) and \
                                            obj_class_list[score_index[k_20]] == gt_obj_class:
                                        self.tp_20 += 1
                                        break



    def evaluate(self):

        list_path = os.path.join(self.list_dirname, (self.subset + "_list.csv"))

        # load list file
        with open(list_path, "r") as f:
            reader = csv.reader(f)
            target_list = [row for row in reader]

        with open(self.pred_filename, 'r') as fobj:
            pred_data = json.load(fobj)
        # Checking format...
        pred_fields = ['version', 'results', 'external_data']
        if not all([field in pred_data.keys() for field in pred_fields]):
            raise IOError('Please input a valid prediction file.')

        # list includes ID(without view_id), activityname

        for one_video in target_list:
            print(target_list)
            video_id = one_video[0]
            one_pred = pred_data['results'][video_id]
            person_id = video_id.split("_")[0]

            ground_truth_filename = os.path.join(self.gt_dirname, person_id, video_id + "_sg.json")
            with open(ground_truth_filename, 'r') as fobj:
                one_gt = json.load(fobj)

            self.compute_one_video_id(one_pred, one_gt)


        if self.num_collection == 0:
            raise IOError('Please input a valid ground truth file.')
        score_10 = self.tp_10 / self.num_collection
        score_20 = self.tp_20 / self.num_collection

        score = (score_10 + score_20) / 2

        if self.verbose:
            print('<<RESULTS>> Home Action Genome Task #1')
            print('\tAccuracy: {:.3g}'.format(score))


